@@ -20,11 +20,36 @@ class MVPolyCube(mvpoly.base.MVPoly) :
     the coefficient array set to the optional *coef*
     argument, a :class:`numpy.ndarray`.
     """
-    def __init__(self, coef = None, **kwd):
-        if coef is None :
-            coef = []
+    def __init__(self, arg=None, **kwd):
+        if isinstance(arg, mvpoly.base.MVPoly) :
+            coef = self.init_from_nonzero(arg, arg.dtype).coef
+            kwd['dtype'] = arg.dtype
+        else :
+            coef = arg
         super(MVPolyCube, self).__init__(**kwd)
-        self._coef = np.array(coef, dtype = self.dtype)
+        if coef is None :
+            coef = np.array([], dtype = self.dtype)
+        else :
+            coef = np.array(coef, dtype = self.dtype)
+        self._coef = coef
+        self.dtype_set_callback(self.dtype)
+
+    @classmethod
+    def init_from_nonzero(self, p, dtype) :
+        """
+        Create a polynomial from a polynomial of any subclass; this is 
+        used internally by the 
+        :py:meth:`~mvpoly.cube.MVPolyCube.__init__` constructor, but
+        may be useful for direct application.
+        """
+        shp  = tuple(1+d for d in p.degrees)
+        nvar = len(shp)
+        coef = np.zeros(shp, dtype=dtype)
+        for pair in p.nonzero :
+            idx, val = pair
+            idx = idx + tuple(0 for _ in range(nvar - len(idx)))
+            coef[idx] = val
+        return MVPolyCube(coef, dtype=dtype)
 
     @classmethod
     def monomials(self, n, **kwd) :
@@ -68,23 +93,71 @@ class MVPolyCube(mvpoly.base.MVPoly) :
 
     @property
     def coef(self) :
+        """
+        The NumPy array of coefficients.
+        """
         return self._coef
 
     @coef.setter
     def coef(self, value) :
         self._coef = value
 
-    def __setitem__(self, key, value) :
-        """
-        The setter method, delegated to the *coef* attribute.
-        """
-        self.coef[key] = value
-
     def __getitem__(self, key) :
         """
-        The getter method, delegated to the *coef* attribute.
+        The get-index method which returns the coefficients of the
+        polynomial by their (zero-based) indices. For the polynomial
+        :math:`p(x,y) = 2x^2 + y + 3` we would access the coefficient of 
+        :math:`x^2` with ``p[2]`` or ``p[2,0]`` or ``p[2,0,0]`` ...
+        (trailing zeros are ignored), the coefficient of 
+        :math:`y` with ``p[0,1]``, ``p[0,1,0]``, ... and the constant
+        term with ``p[()]``, ``p[0]``, ``p[0,0]``, ... (Note that 
+        ``p[]`` is Python syntax error).  Indices outside the shape of
+        the underlying NumPy array will return zero (without changing
+        shape of the array).
+
+        Note that slice indexing will not work here, ``p[1:3]`` will
+        result in an error: instead use the 
+        :py:meth:`~mvpoly.cube.MVPolyCube.coef` 
+        method to access the underlying array, ``p.coef[1:3]``.
         """
-        return self.coef[key]
+        key1d  = np.atleast_1d(key)
+        coef1d = np.atleast_1d(self.coef)
+        nvar = len(coef1d.shape)
+        narg = len(key1d)
+        if narg > nvar :
+            if (key1d[nvar:] == 0).all() :
+                key1d = key1d[:nvar]
+            else :
+                return 0
+        elif narg < nvar :
+            zeropad = np.zeros(nvar-narg, dtype=int)
+            key1d = np.concatenate((key1d, zeropad), axis=1)
+        try :
+            val = coef1d[tuple(key1d)]
+        except IndexError : 
+            val = 0
+        return val
+
+    def __setitem__(self, key, value) :
+        """
+        As :py:meth:`~mvpoly.cube.MVPolyCube.__getitem__`, 
+        but setting the coefficient. The underlying array is 
+        grown to accomodate the new coefficient if needed.
+        """
+        A = self.coef
+        # coerce dimensional agreement
+        dd = len(A.shape) - len(key)
+        if dd > 0 :
+            key = key + tuple(0 for _ in range(dd)) 
+        elif dd < 0 :
+            newshape = mvpoly.util.cube.dimension_pad(A.shape, len(key))
+            A = np.reshape(A, newshape)
+        # minimal shape which contains A and the key
+        newshape = tuple(max(A.shape[n], key[n]+1) for n in range(len(key)))
+        if newshape != A.shape :
+            A = mvpoly.util.cube.array_enlarge(A, newshape)
+        A[key] = value
+        self.coef = A
 
     def __add__(self, other) :
         """
@@ -144,6 +217,21 @@ class MVPolyCube(mvpoly.base.MVPoly) :
         """
         return ((self - other).coef == 0).all()
 
+    @property
+    def nonzero(self) :
+        """
+        Returns a list of 2-element tuples, for each the first
+        being a (monomial) index, the second the corresponding 
+        coefficient. The indices have trailing zeros, so the return 
+        value for the polynomial
+        :math:`p(x,y) = 2 + 3x + 4y` would be
+        ``[((0,0), 2), ((1,0), 3), ((0,1), 4)]``.
+        """
+        dt = self.dtype
+        nzidx = np.nonzero(self.coef)
+        return zip((tuple(idx) for idx in np.transpose(nzidx)), 
+                   (np.array(c, dtype=dt) for c in self.coef[nzidx]))
+
     @property
     def degrees(self) :
         """
@@ -244,10 +332,10 @@ class MVPolyCube(mvpoly.base.MVPoly) :
 
     def int(self, *args, **kwargs) :
         """
-        Indefinite integral of polynomial. The arguments are
-        as for :meth:`diff`, but an optional *dtype* keyword 
-        argument can be appended to specify the *dtype* of the
-        output polynomial.
+        Indefinite integral of polynomial. The arguments are as for 
+        :py:meth:`~mvpoly.cube.MVPolyCube.diff`, 
+        but an optional *dtype* keyword argument can be appended to 
+        specify the *dtype* of the output polynomial.
         """
         dtype = np.dtype(kwargs.get('dtype', self.dtype))
         return MVPolyCube(mvpoly.util.cube.int(self.coef, args, dtype), 
