@@ -23,11 +23,30 @@ class MVPolyDict(mvpoly.base.MVPoly) :
     the coefficient dictionary set to the optional *coef*
     argument.
     """
-    def __init__(self, coef = None, **kwd):
+    def __init__(self, arg=None, **kwd) :
+        if isinstance(arg, mvpoly.base.MVPoly) :
+            coef = self.init_from_nonzero(arg, arg.dtype).coef
+            kwd['dtype'] = arg.dtype
+        else :
+            coef = arg
         super(MVPolyDict, self).__init__(**kwd)
-        self._coef = {} if coef is None else coef
+        if coef is None :
+            coef = {}
+        self._coef = coef
         self.dtype_set_callback(self.dtype)
 
+    @classmethod
+    def init_from_nonzero(self, p, dtype) :
+        """
+        Create a polynomial from one of any subclass; this is used 
+        internally by the 
+        :py:meth:`~mvpoly.cube.MVPolyDict.__init__` constructor, but
+        may be useful for direct application.
+        """
+        q = self.zero(dtype = dtype)
+        for idx, val in p.nonzero :
+            q[idx] = val
+        return q
 
     @classmethod
     def monomials(self, n, **kwd) :
@@ -75,6 +94,9 @@ class MVPolyDict(mvpoly.base.MVPoly) :
             
     @property
     def coef(self) :
+        """
+        Return the dictionary of coefficients. 
+        """
         return self._coef
 
     @coef.setter
@@ -84,7 +106,8 @@ class MVPolyDict(mvpoly.base.MVPoly) :
 
     def __setitem__(self, index, value) :
         """
-        The setter method, delegated to the *coef* attribute.
+        The setter method, which sets coefficient of the
+        given *index* to the given *value*.
         """
         if value == 0 :
             return
@@ -94,7 +117,8 @@ class MVPolyDict(mvpoly.base.MVPoly) :
 
     def __getitem__(self, index) :
         """
-        The getter method, delegated to the *coef* attribute.
+        The getter method, returning the coefficient of the
+        specified *index*.
         """
         key = MVPolyDictMonomial(index = index).key
         return self.coef.get(key, np.array(0, self.dtype))
@@ -226,6 +250,19 @@ class MVPolyDict(mvpoly.base.MVPoly) :
                     k -= 1
         return s[0]
 
+    @property
+    def nonzero(self) :
+        """
+        Returns a list of 2-element tuples, for each the first
+        being a (monomial) index, the second the corresponding 
+        coefficient. The indices do not have trailing zeros,
+        so the return value for the polynomial
+        :math:`p(x,y) = 2 + 3x + 4y` would be
+        ``[((), 2), ((1,), 3), ((0,1), 4)]``.
+        """
+        return [(MVPolyDictMonomial(key = k).index, v) 
+                for k, v in self.coef.iteritems()]
+
     @property
     def degrees(self) :
         """
@@ -237,10 +274,10 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         n = mvpoly.util.common.max_or_default((len(idx) for idx in idxs), 0)
         idxs = [idx + (0,) * (n - len(idx)) for idx in idxs]
         if idxs == [] :
-            degs = [] 
+            degs = () 
         else :
-            degs = [max(idx[i] for idx in idxs) for i in range(n)] 
-        return tuple(degs)
+            degs = tuple(max(idx[i] for idx in idxs) for i in range(n)) 
+        return degs
 
     @property
     def degree(self) :
@@ -409,7 +446,7 @@ class MVPolyDictMonomial(object) :
         A hashable representation (as a tuple of pairs) of the
         dictionary.
         """
-        return tuple([(k, self.dict[k]) for k in sorted(self.dict.keys())])
+        return tuple((k, self.dict[k]) for k in sorted(self.dict.keys()))
 
     @key.setter
     def key(self, kvs) :
@@ -439,7 +476,7 @@ class MVPolyDictMonomial(object) :
         thus ``m.index_of_length(3)`` would return ``(3, 2, 0)``
         if ``m`` represents the monomial :math:`x^3y^2`.
         """
-        return tuple([self.dict.get(k, 0) for k in range(n)])
+        return tuple(self.dict.get(k, 0) for k in range(n))
 
     @property
     def occurences(self) :
@@ -464,6 +501,8 @@ class MVPolyDictMonomial(object) :
         """
         Return the product of monomials.
         """
-        d = mvpoly.util.dict.merge_dicts(self.dict, other.dict, lambda x, y : x+y) 
+        d = mvpoly.util.dict.merge_dicts(self.dict, 
+                                         other.dict, 
+                                         lambda x, y : x+y) 
         return MVPolyDictMonomial(dict = d)
         
