@@ -52,6 +52,19 @@ class TestMVPolyCubeDtype(unittest.TestCase) :
         p.dtype = complex
         check_dtype(self, p, complex)
 
+class TestMVPolyCubeSetitem(unittest.TestCase) :
+
+    def test_regression_broadcast(self) :
+        # regression in __setitem__() : when passed an empty
+        # tuple the value was broadcast to the whole array,
+        # we don't want that
+        coef = np.zeros((3,3), dtype=int)
+        p = MVPolyCube(coef, dtype=int)
+        p[()] = 1
+        q = MVPolyCube.one(dtype=int)
+        self.assertTrue(p == q, "setitem broadcast regression")
+
+
 class TestMVPolyCubeEqual(unittest.TestCase) :
 
     def test_equal_self(self) :
@@ -75,6 +88,17 @@ class TestMVPolyCubeEqual(unittest.TestCase) :
         self.assertTrue(p != q, "bad non-equality")
 
 
+class TestMVPolyCubeNonzero(unittest.TestCase) :
+
+    def test_nonzero_enumerate(self) :
+        x, y = MVPolyCube.monomials(2)
+        p = 3*x + y**2 + 7
+        obt = p.nonzero
+        exp = [((0, 0), 7.0), ((1, 0), 3.0), ((0, 2), 1.0)]
+        self.assertTrue(sorted(obt) == sorted(exp),
+                        "nonzero")
+
+
 class TestMVPolyCubeAdd(unittest.TestCase) :
 
     def setUp(self) :
@@ -223,26 +247,6 @@ class TestMVPolyCubeMonomials(unittest.TestCase) :
                             "bad type: \n%s" % repr(m.coef.dtype))
 
 
-class TestMVPolyCubeGetSet(unittest.TestCase) :
-
-    def setUp(self) :
-        x, y = MVPolyCube.monomials(2)
-        self.p = 2*x**2 + 3*x*y + 1
-
-    def test_getitem(self) :
-        obtained = self.p[0:2,:]
-        expected = [[1, 0], [0, 3]]
-        self.assertTrue((obtained == expected).all(),
-                        "bad getitem:\n%s" % repr(obtained))
-
-    def test_setitem(self) :
-        self.p[0, :] = [2, 2]
-        obtained = self.p.coef
-        expected = [[2, 2], [0, 3], [2, 0]]
-        self.assertTrue((obtained == expected).all(),
-                        "bad getitem:\n%s" % repr(obtained))
-
-
 class TestMVPolyCubeNeg(unittest.TestCase) :
 
     def test_negation(self) :
