@@ -222,6 +222,17 @@ class TestMVPolyDictSetget(unittest.TestCase) :
                         "bad get unset:\n%s" % repr(obt))
 
 
+class TestMVPolyDictNonzero(unittest.TestCase) :
+
+    def test_nonzero_enumerate(self) :
+        x, y = MVPolyDict.monomials(2)
+        p = 3*x + y**2 + 7
+        obt = p.nonzero
+        exp = [((), 7.0), ((1,), 3.0), ((0, 2), 1.0)]
+        self.assertTrue(sorted(obt) == sorted(exp),
+                        "nonzero")
+
+
 class TestMVPolyDictAddSub(unittest.TestCase) :
 
     def setUp(self) :
